/***************************************************************************
 *   Copyright (C) 2009 - 2010 by Dr. Oliver Heidbüchel                    *
 *                         (post@oliver-heidbuechel.de)                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "sparqlquerytab.h"

#include <Soprano/Soprano>
 
#include <KAction>
#include <KLocale>
#include <KMenu>
#include <KMessageBox>
#include <KPushButton>

#include <QAction>
#include <QApplication>
#include <QBoxLayout>
#include <QContextMenuEvent>
#include <QClipboard>
#include <QSizePolicy>
#include <QSplitter>
#include <QString>
#include <QStringList>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QWidget>

 
SparqlQueryTab::SparqlQueryTab(Soprano::Model *m, QWidget *parent)    
  : QSplitter(parent)
{
  // Model
  model = m;
  
  // Splitter
  setOrientation(Qt::Vertical);
  setChildrenCollapsible(false);
  
  // upper part
  QWidget * upperContainer = new QWidget(this);
  QHBoxLayout * layout = new QHBoxLayout(upperContainer);
  queryField = new KTextEdit(upperContainer);
  queryField->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred));
  layout->addWidget(queryField);
  KPushButton * b = new KPushButton(i18n("Execute Query"), upperContainer);
  layout->addWidget(b);
  addWidget(upperContainer);
  
  //lower part
  resultField = new QTableWidget(0, 0, this);
  addWidget(resultField);

  // Connections
  connect(b, SIGNAL(clicked()), this, SLOT(query()));
}

void SparqlQueryTab::query(){
  QString query = queryField->toPlainText();
  if (query.compare("") != 0){
    Soprano::QueryResultIterator it = model->executeQuery(query, Soprano::Query::QueryLanguageSparql, "SPARQL");
    QStringList bindings = it.bindingNames();
    int number = bindings.count();
    resultField->setColumnCount(number);
    resultField->setRowCount(0);
    for (int i = 0; i < number; i++){
      resultField->setHorizontalHeaderItem(i, new QTableWidgetItem("?" + bindings.value(i)));
    }
    while( it.next() ) {
      int zeile = resultField->rowCount();
      resultField->insertRow(zeile);
      for (int i = 0; i < number; i++){
	resultField->setItem(zeile, i, new QTableWidgetItem(it.binding(bindings.value(i)).toN3(), QTableWidgetItem::Type));
	resultField->item(zeile, i)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
      }
    }
    if( it.lastError() ) {
      KMessageBox::error(this, i18n("Query failed:\n") + model->lastError().message(), i18n("Query failed!"));
    }
    it.close();
  }
}

void SparqlQueryTab::executeQuery(const QString & queryString){
  queryField->setText(queryString);
  query();
}
 
void SparqlQueryTab::contextMenuEvent(QContextMenuEvent *event)
{
  QTableWidgetItem * current = resultField->currentItem();
  if (current != 0){
    QString itemText = current->text();
  
    KMenu contextMenu(this);
    QAction * copyAction = contextMenu.addAction(KIcon("edit-copy"), i18nc("@action:inmenu", "Copy"));
    contextMenu.addSeparator();
    QAction * subjectQueryAction = contextMenu.addAction(i18nc("@action:inmenu", "Subject-Query"));
    QAction * predicateQueryAction = contextMenu.addAction(i18nc("@action:inmenu", "Predicate-Query"));
    QAction * objectQueryAction = contextMenu.addAction(i18nc("@action:inmenu", "Object-Query"));
    QAction * contextQueryAction = contextMenu.addAction(i18nc("@action:inmenu", "Context-Query"));
    contextMenu.addSeparator();
    QAction * asSubjectAction = contextMenu.addAction(i18nc("@action:inmenu", "As Subject"));
    QAction * asPredicateAction = contextMenu.addAction(i18nc("@action:inmenu", "As Predicate"));
    QAction * asObjectAction = contextMenu.addAction(i18nc("@action:inmenu", "As Object"));
    QAction * asContextAction = contextMenu.addAction(i18nc("@action:inmenu", "As Context"));
    
    QAction * choosenAction = contextMenu.exec(QCursor::pos());
    if (choosenAction == asSubjectAction){
      emit asSubjectSelected(itemText);
    } else if (choosenAction == asPredicateAction){
      emit asPredicateSelected(itemText);
    } else if (choosenAction == asObjectAction){
      emit asObjectSelected(itemText);
    } else if (choosenAction == asContextAction){
      emit asContextSelected(itemText);
    } else if (choosenAction == subjectQueryAction){
      QString query = "select distinct ?subject ?predicate ?object ?context where { graph ?context { " + itemText + " ?predicate ?object . ?subject ?predicate ?object . } }";
      emit newQueryCommisioned(query);
    } else if (choosenAction == predicateQueryAction){
      QString query = "select distinct ?subject ?predicate ?object ?context where { graph ?context { ?subject " + itemText + " ?object . ?subject ?predicate ?object . } }";
      emit newQueryCommisioned(query);
    } else if (choosenAction == objectQueryAction){
      QString query = "select distinct ?subject ?predicate ?object ?context where { graph ?context { ?subject ?predicate " + itemText + " . ?subject ?predicate ?object . } }";
      emit newQueryCommisioned(query);
    } else if (choosenAction == contextQueryAction){
      QString query = "select distinct ?subject ?predicate ?object ?context where { graph " +itemText + " { ?subject ?predicate ?object . } . graph ?context { ?subject ?predicate ?object . } }";
      emit newQueryCommisioned(query);
    } else if (choosenAction == copyAction){
      QApplication::clipboard()->setText(itemText);
    }
  }
}







